CONN_UNIFIED_MAPPINGS = {
    "db2": {"datasource_type": "db2", "credential_key": "[db2-conn|cloud]", "label": "IBM Db2"},
    "db2cloud": {"datasource_type": "db2cloud", "credential_key": "[db2cloud-conn]", "label": "IBM Db2 on Cloud"},
    "bigsql": {"datasource_type": "bigsql", "credential_key": "[db2BigSql|cloud]", "label": "IBM Db2 Big SQL"},
    "db2iseries": {"datasource_type": "db2iseries", "credential_key": "[db2I-conn|cloud]", "label": "IBM Db2 for i"},
    "db2zos": {"datasource_type": "db2zos", "credential_key": "[db2zos-conn|cloud]", "label": "IBM Db2 for z/OS"},
    "amazons3": {"datasource_type": "amazons3", "credential_key": "[s3-amazon]", "label": "Amazon S3"},
    "mysql": {"datasource_type": "mysql", "credential_key": "[mysql-conn|cloud]", "label": "MySQL"},
    "datastax": {
        "datasource_type": "datastax",
        "credential_key": "[datastax-conn|cloud]",
        "label": "DataStax Enterprise",
    },
    "informix": {"datasource_type": "informix", "credential_key": "[Informix-conn|cloud]", "label": "IBM Informix"},
    "match360": {"datasource_type": "match360", "credential_key": "[match360-conn]", "label": "IBM Match 360"},
    "dvm": {
        "datasource_type": "dvm",
        "credential_key": "[dvm|cloud]",
        "label": "IBM Data Virtualization Manager for z/OS",
    },
    "dv": {"datasource_type": "dv", "credential_key": "[dv-onPrem|onPrem]", "label": "IBM Data Virtualization"},
    "generics3": {"datasource_type": "generics3", "credential_key": "[s3-generic]", "label": "Generic S3"},
    "box": {"datasource_type": "box", "credential_key": "[box-conn]", "label": "Box"},
    "elasticsearch": {
        "datasource_type": "elasticsearch",
        "credential_key": "[elasticsearch-conn]",
        "label": "Elasticsearch",
    },
    "dropbox": {"datasource_type": "dropbox", "credential_key": "[dropbox-conn]", "label": "Dropbox"},
    "ftp": {"datasource_type": "ftp", "credential_key": "[ftp-conn|cloud]", "label": "FTP"},
    "tableau": {"datasource_type": "tableau", "credential_key": "[tableau-cloud-conn]", "label": "Tableau"},
    "cassandra": {
        "datasource_type": "cassandra",
        "credential_key": "[cassandra-conn|cloud]",
        "label": "Apache Cassandra",
    },
    "impala": {"datasource_type": "impala", "credential_key": "[impala-conn|cloud]", "label": "Apache Impala"},
    "derby": {"datasource_type": "derby", "credential_key": "[apacheDerby-conn|cloud]", "label": "Apache Derby"},
    "bigquery": {"datasource_type": "bigquery", "credential_key": "[bigquery-conn]", "label": "Google BigQuery"},
    "azuredatalake": {
        "datasource_type": "azuredatalake",
        "credential_key": "[azure-datalake-conn]",
        "label": "Microsoft Azure Data Lake Storage",
    },
    "azuresql": {
        "datasource_type": "azuresql",
        "credential_key": "[azure-sql-conn]",
        "label": "Microsoft Azure SQL Database",
    },
    "sqlserver": {
        "datasource_type": "sqlserver",
        "credential_key": "[sqlServer-conn|cloud]",
        "label": "Microsoft SQL Server",
    },
    "sapodata": {"datasource_type": "sapodata", "credential_key": "[sapodata|onPrem]", "label": "SAP OData"},
    "sybase": {"datasource_type": "sybase", "credential_key": "[SAPASE-conn|cloud]", "label": "SAP ASE"},
    "mariadb": {"datasource_type": "mariadb", "credential_key": "[mariadb-conn]", "label": "MariaDB"},
    "postgresql": {"datasource_type": "postgresql", "credential_key": "[postgres-conn]", "label": "PostgreSQL"},
    "greenplum": {"datasource_type": "greenplum", "credential_key": "[greenplum|cloud]", "label": "Greenplum"},
    "oracle": {"datasource_type": "oracle", "credential_key": "[oracle-conn|cloud]", "label": "Oracle"},
    "dremio": {"datasource_type": "dremio", "credential_key": "[Dremio|cloud]", "label": "Dremio"},
    "mongodb": {"datasource_type": "mongodb", "credential_key": "[mongodb-conn]", "label": "MongoDB"},
    "presto": {"datasource_type": "presto", "credential_key": "[presto]", "label": "Presto"},
    "teradata": {"datasource_type": "teradata", "credential_key": "[teradata|cloud]", "label": "Teradata"},
    "netezza": {
        "datasource_type": "netezza",
        "credential_key": "[netezza-conn|cloud]",
        "label": "IBM Netezza Performance Server",
    },
    "denodo": {"datasource_type": "denodo", "credential_key": "[denodo-conn|cloud]", "label": "Denodo"},
    "vertica": {"datasource_type": "vertica", "credential_key": "[vertica-conn]", "label": "Vertica"},
    "snowflake": {"datasource_type": "snowflake", "credential_key": "[snowflake-conn]", "label": "Snowflake"},
    "singlestore": {"datasource_type": "singlestore", "credential_key": "[SingleStore]", "label": "SingleStoreDB"},
    "salesforce": {"datasource_type": "salesforce", "credential_key": "[salesforce-conn]", "label": "Salesforce.com"},
    "azuresynapse": {
        "datasource_type": "azuresynapse",
        "credential_key": "[azure-synapse-conn]",
        "label": "Microsoft Azure Synapse Analytics",
    },
    "db2fordatastage": {
        "datasource_type": "db2-datastage",
        "credential_key": "[db2datastage|cloud]",
        "label": "IBM Db2 for DataStage",
    },
    "amazon_redshift": {"datasource_type": "redshift", "credential_key": "[redshift-conn]", "label": "Amazon Redshift"},
    "mysql_amazon": {
        "datasource_type": "mysql-amazon",
        "credential_key": "[rds-mysql-conn]",
        "label": "Amazon RDS for MySQL",
    },
    "cloud_object_storage": {
        "datasource_type": "cloudobjectstorage",
        "credential_key": "[cos-conn]",
        "label": "IBM Cloud Object Storage",
    },
    "cognos_analytics": {
        "datasource_type": "cognos-analytics",
        "credential_key": "[cognos-conn]",
        "label": "IBM Cognos Analytics",
    },
    "mongodb_ibmcloud": {
        "datasource_type": "mongodb-ibmcloud",
        "credential_key": "[mongodb-conn]",
        "label": "IBM Cloud Databases for MongoDB",
    },
    "hdfs_apache": {
        "datasource_type": "hdfs-apache",
        "credential_key": "[HDFS-SSL-conn|cloud]",
        "label": "Apache HDFS",
    },
    "apache_kafka": {"datasource_type": "apachekafka", "credential_key": "[Kafka-conn]", "label": "Apache Kafka"},
    "cassandra_datastage": {
        "datasource_type": "cassandra-datastage",
        "credential_key": "[cassandraDS-conn|cloud]",
        "label": "Apache Cassandra for DataStage",
    },
    "google_cloud_storage": {
        "datasource_type": "googlecloudstorage",
        "credential_key": "[googlecos-conn]",
        "label": "Google Cloud Storage",
    },
    "google_pub_sub": {
        "datasource_type": "googlepubsub",
        "credential_key": "[pubsub-conn]",
        "label": "Google Cloud Pub/Sub",
    },
    "azure_blob_storage": {
        "datasource_type": "azureblobstorage",
        "credential_key": "[azure-blob-storage-conn]",
        "label": "Microsoft Azure Blob Storage",
    },
    "azure_cosmos": {
        "datasource_type": "cosmos",
        "credential_key": "[cosmos-conn]",
        "label": "Microsoft Azure Cosmos DB",
    },
    "azure_databricks": {
        "datasource_type": "databricks",
        "credential_key": "[azure-databricks-conn]",
        "label": "Microsoft Azure Databricks",
    },
    "azure_file_storage": {
        "datasource_type": "azurefilestorage",
        "credential_key": "[azure-file-storage-conn]",
        "label": "Microsoft Azure File Storage",
    },
    "sapiq": {"datasource_type": "sybaseiq", "credential_key": "[SAP-IQ-conn|cloud]", "label": "SAP IQ"},
    "postgresql_ibmcloud": {
        "datasource_type": "postgresql-ibmcloud",
        "credential_key": "[db-for-postgres-conn]",
        "label": "IBM Cloud Databases for PostgreSQL",
    },
    "teradata_datastage": {
        "datasource_type": "teradata-datastage",
        "credential_key": "[TeradataOptimized|cloud]",
        "label": "Teradata database for DataStage",
    },
    "oracle_datastage": {
        "datasource_type": "oracle-datastage",
        "credential_key": "[OracleDs|cloud]",
        "label": "Oracle Database for DataStage",
    },
    "apache_hive": {"datasource_type": "hive", "credential_key": "[hive-conn|cloud]", "label": "Apache Hive"},
    "azure_postgresql": {
        "datasource_type": "postgresql-azure",
        "credential_key": "[azure-postgres-conn]",
        "label": "Azure PostgreSQL",
    },
    "watsonx_data": {
        "datasource_type": "lakehouse",
        "credential_key": "[watsonxData-conn]",
        "label": "IBM watsonx.data Presto",
    },
    "planning_analytics": {
        "datasource_type": "tm1odata",
        "credential_key": "[ibm-planning-analytics-conn|cloud]",
        "label": "IBM Planning Analytics",
    },
    "odbc": {"datasource_type": "odbc-datastage", "credential_key": "[odbcDb2-conn|cloud]", "label": "ODBC"},
    "ibm_mq": {
        "datasource_type": "webspheremq-datastage",
        "credential_key": "[websphere-tcp-conn|cloud]",
        "label": "IBM MQ",
    },
    "amazonrds_oracle": {
        "datasource_type": "oracle-amazon",
        "credential_key": "[rds-oracle-conn]",
        "label": "Amazon RDS for Oracle",
    },
    "db2warehouse": {
        "datasource_type": "dashdb",
        "credential_key": "[DB2_Warehouse-conn]",
        "label": "IBM Db2 Warehouse",
    },
    "amazon_postgresql": {
        "datasource_type": "postgresql-amazon",
        "credential_key": "[PostgresSQL-AmazonRDS-conn]",
        "label": "Amazon RDS for PostgreSQL",
    },
    "google_looker": {"datasource_type": "looker", "credential_key": "[looker-conn]", "label": "Google Looker"},
    "netezza_optimized": {
        "datasource_type": "netezza-datastage",
        "credential_key": "[netezza-conn|cloud]",
        "label": "IBM Netezza Performance Server for DataStage",
    },
    "jdbc": {"datasource_type": "jdbc", "credential_key": "", "label": "Generic JDBC"},
    "http": {"datasource_type": "http", "credential_key": "", "label": "HTTP"},
    "salesforceapi": {
        "datasource_type": "salesforce-datastage",
        "credential_key": "",
        "label": "Salesforce API for DataStage",
    },
}



from ibm_watsonx_data_integration.services.datastage.models.enums import AMAZONRDS_ORACLE_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAmazonRdsOracleConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "amazonrdsoracle_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _autotest_1_ = fc.add_stage("Amazon RDS for Oracle", "AUTOTEST_1")
        _autotest_1_.configuration.schema_name = "TM_DS_1"
        _autotest_1_.configuration.table_name = "AUTOTEST"

        _autotest_1_.configuration.connection.connection_mode = "service_name"
        _autotest_1_.configuration.connection.hostname_or_ip_address = environ["AMAZONRDSORACLE_HOST"]
        _autotest_1_.configuration.connection.metadata_discovery = (
            AMAZONRDS_ORACLE_CONNECTION.MetadataDiscovery.no_remarks
        )
        _autotest_1_.configuration.connection.password = environ["AMAZONRDSORACLE_PASSWORD"]
        _autotest_1_.configuration.connection.port = "1521"
        _autotest_1_.configuration.connection.proxy = False
        _autotest_1_.configuration.connection.service_name = "DATABASE"
        _autotest_1_.configuration.connection.port_is_ssl_enabled = False
        _autotest_1_.configuration.connection.username = environ["AMAZONRDSORACLE_USERNAME"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        autotest_1_schema = _autotest_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        autotest_1_schema.add_field("NCHAR", "col1").length(6)
        autotest_1_schema.add_field("DECIMAL", "col2").length(6).precision(6)
        autotest_1_schema.add_field("NVARCHAR", "col3").nullable().length(6)
        autotest_1_schema.add_field("DOUBLE", "col4")
        autotest_1_schema.add_field("DECIMAL", "col6").length(19).precision(19)
        autotest_1_schema.add_field("VARBINARY", "col7").length(6)
        autotest_1_schema.add_field("DOUBLE", "col8")
        autotest_1_schema.add_field("DECIMAL", "col9").length(38).precision(38)
        autotest_1_schema.add_field("TIMESTAMP", "col10")
        autotest_1_schema.add_field("DECIMAL", "col11").length(38).precision(38)
        autotest_1_schema.add_field("LONGNVARCHAR", "col12").length(1024)
        autotest_1_schema.add_field("LONGNVARCHAR", "col13").length(1024)
        autotest_1_schema.add_field("NCHAR", "col15").length(6)
        autotest_1_schema.add_field("DECIMAL", "col16").length(6).precision(6)
        autotest_1_schema.add_field("NVARCHAR", "col17").nullable().length(6)
        autotest_1_schema.add_field("REAL", "col18").length(7)
        autotest_1_schema.add_field("DECIMAL", "col19").length(38).precision(38)
        autotest_1_schema.add_field("TIMESTAMP", "col20")
        autotest_1_schema.add_field("TIMESTAMP", "col21")
        autotest_1_schema.add_field("DECIMAL", "col22").length(3).precision(3)
        autotest_1_schema.add_field("NVARCHAR", "col24").nullable().length(6)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from pydantic import ConfigDict, Field
from typing import ClassVar


class NetezzaOptimizedConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "63e2d853-e650-3b59-91a5-95e7bf725b9b"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    cas_lite_service_authorization_header: str | None = Field(None, alias="cas_lite_auth_header")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    database: str = Field(None, alias="database")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    hostname: str = Field(None, alias="hostname")
    password: str = Field(None, alias="password")
    port: int = Field(5480, alias="port")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    original_hostname_of_the_resource: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    ssl_certificate_pem: str | None = Field(None, alias="ssl_certificate")
    ssl_connection: bool | None = Field(False, alias="ssl_connection")
    use_cas_lite_service: bool | None = Field(True, alias="use_cas_lite")
    use_separate_connection_for_twt: bool | None = Field(False, alias="use_separate_connection_for_twt")
    twt_separate_connection_database_name: str = Field(None, alias="use_separate_connection_for_twt.database")
    twt_separate_connection_password: str | None = Field(None, alias="use_separate_connection_for_twt.password")
    user_name: str | None = Field(None, alias="use_separate_connection_for_twt.username")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("ssl_certificate_pem") if (self.ssl_connection) else exclude.add("ssl_certificate_pem")
        include.add("password") if (not self.defer_credentials) else exclude.add("password")
        include.add("user_name") if (self.use_separate_connection_for_twt) else exclude.add("user_name")
        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("twt_separate_connection_database_name")
            if (self.use_separate_connection_for_twt)
            else exclude.add("twt_separate_connection_database_name")
        )
        (
            include.add("twt_separate_connection_password")
            if (self.use_separate_connection_for_twt)
            else exclude.add("twt_separate_connection_password")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        include.add("username") if (not self.defer_credentials) else exclude.add("username")
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")

        (
            include.add("twt_separate_connection_password")
            if (self.use_separate_connection_for_twt == "true" or self.use_separate_connection_for_twt)
            else exclude.add("twt_separate_connection_password")
        )
        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service == "true" or self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        (
            include.add("twt_separate_connection_database_name")
            if (self.use_separate_connection_for_twt == "true" or self.use_separate_connection_for_twt)
            else exclude.add("twt_separate_connection_database_name")
        )
        include.add("password") if not self.defer_credentials else exclude.add("password")
        (
            include.add("user_name")
            if (self.use_separate_connection_for_twt == "true" or self.use_separate_connection_for_twt)
            else exclude.add("user_name")
        )
        include.add("username") if not self.defer_credentials else exclude.add("username")
        (
            include.add("ssl_certificate_pem")
            if (self.ssl_connection == "true" or self.ssl_connection)
            else exclude.add("ssl_certificate_pem")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



import warnings
from ibm_watsonx_data_integration.services.datastage.models.base_stage import BaseStage
from ibm_watsonx_data_integration.services.datastage.models.enums import WEB_SERVICE
from pydantic import Field
from typing import ClassVar


class web_service(BaseStage):

    op_name: ClassVar[str] = "WSTransformerPX"
    node_type: ClassVar[str] = "execution_node"
    image: ClassVar[str] = "/data-intg/flows/graphics/palette/WSTransformerPX.svg"
    label: ClassVar[str] = "Web Service"
    allow_column_mapping: bool | None = Field(False, alias="allow_column_mapping")
    authentication_password: str | None = Field("", alias="authentication_password")
    authentication_user_name: str | None = Field("", alias="authentication_user_name")
    auto_column_propagation: bool | None = Field(None, alias="auto_column_propagation")
    buf_free_run: int | None = Field(50, alias="buf_free_run")
    buf_free_run_ronly: int | None = Field(50, alias="buf_free_run_ronly")
    buf_mode: WEB_SERVICE.BufMode | None = Field(WEB_SERVICE.BufMode.default, alias="buf_mode")
    buf_mode_ronly: WEB_SERVICE.BufModeRonly | None = Field(WEB_SERVICE.BufModeRonly.default, alias="buf_mode_ronly")
    certificate_passwd: str | None = Field("", alias="certificate_passwd")
    coll_type: WEB_SERVICE.CollType | None = Field(WEB_SERVICE.CollType.auto, alias="coll_type")
    combinability: WEB_SERVICE.Combinability | None = Field(WEB_SERVICE.Combinability.auto, alias="combinability")
    current_output_link_type: str = Field("PRIMARY", alias="currentOutputLinkType")
    disk_write_inc: int | None = Field(1048576, alias="disk_write_inc")
    disk_write_inc_ronly: int | None = Field(1048576, alias="disk_write_inc_ronly")
    enable_flow_acp_control: bool = Field(True, alias="enableFlowAcpControl")
    enable_schemaless_design: bool = Field(False, alias="enableSchemalessDesign")
    error_handling: WEB_SERVICE.ErrorHandling | None = Field(WEB_SERVICE.ErrorHandling.fatal, alias="error_handling")
    execmode: WEB_SERVICE.Execmode | None = Field(WEB_SERVICE.Execmode.default_seq, alias="execmode")
    field_delimiter: str | None = Field(" ", alias="field_delimiter")
    flow_dirty: str | None = Field("false", alias="flow_dirty")
    has_reference_output: bool | None = Field(False, alias="hasReferenceOutput")
    hide: bool | None = Field(False, alias="hide")
    inherit_from_stage: int | None = Field(1, alias="inherit_from_stage")
    input_col: list | None = Field(None, alias="input_col")
    input_count: int | None = Field(0, alias="input_count")
    input_description: str | None = Field("0", alias="input_description")
    input_link_description: list | None = Field("", alias="inputLinkDescription")
    input_message_table_id: str | None = Field("", alias="input_message_table_id")
    input_name: str | None = Field("", alias="input_name")
    input_namespace_name: str | None = Field("", alias="input_namespace_name")
    input_namespace_value: str | None = Field("", alias="input_namespace_value")
    input_request: str | None = Field("", alias="input_request")
    inputcol_properties: list | None = Field([], alias="inputcolProperties")
    is_authentication_required: bool | None = Field(False, alias="is_authentication_required")
    is_automatic_trust: bool | None = Field(False, alias="is_automatic_trust")
    is_certificate_required: bool | None = Field(False, alias="is_certificate_required")
    is_input_header_xml_column: bool | None = Field(False, alias="is_input_header_xml_column")
    is_input_xml_column: bool | None = Field(False, alias="is_input_xml_column")
    is_log_reject_reason_disabled: bool | None = Field(False, alias="is_log_reject_reason_disabled")
    is_output_header_xml_column: bool | None = Field(False, alias="is_output_header_xml_column")
    is_output_xml_column: bool | None = Field(False, alias="is_output_xml_column")
    is_pass_through_disabled: bool | None = Field(False, alias="is_pass_through_disabled")
    is_proxy_required: bool | None = Field(False, alias="is_proxy_required")
    is_reject_column: bool | None = Field(False, alias="is_reject_column")
    is_reject_link: bool | None = Field(False, alias="is_reject_link")
    jvm_options: str | None = Field("", alias="jvm_options")
    key_col_select: WEB_SERVICE.KeyColSelect | None = Field(WEB_SERVICE.KeyColSelect.default, alias="keyColSelect")
    key_cols_coll: list | None = Field([], alias="keyColsColl")
    key_cols_coll_ordered: list | None = Field([], alias="keyColsCollOrdered")
    key_cols_coll_robin: list | None = Field([], alias="keyColsCollRobin")
    key_cols_none: list | None = Field([], alias="keyColsNone")
    key_cols_part: list | None = Field([], alias="keyColsPart")
    keystore_file: str | None = Field("", alias="keystore_file")
    link_minimised: int | None = Field(0, alias="LinkMinimised")
    log_response: WEB_SERVICE.LogResponse | None = Field(WEB_SERVICE.LogResponse.zero, alias="log_response")
    max_mem_buf_size: int | None = Field(3145728, alias="max_mem_buf_size")
    max_mem_buf_size_ronly: int | None = Field(3145728, alias="max_mem_buf_size_ronly")
    operation_timeout: int | None = Field(0, alias="operation_timeout")
    output_acp_should_hide: bool = Field(True, alias="outputAcpShouldHide")
    output_col: list | None = Field(None, alias="output_col")
    output_count: int | None = Field(0, alias="output_count")
    output_link_description: list | None = Field("", alias="outputLinkDescription")
    output_message_table_id: str | None = Field("", alias="output_message_table_id")
    output_namespace_name: str | None = Field("", alias="output_namespace_name")
    output_namespace_value: str | None = Field("", alias="output_namespace_value")
    output_reject_column: str | None = Field("", alias="output_reject_column")
    outputcol_properties: list | None = Field([], alias="outputcolProperties")
    part_client_dbname: str | None = Field(None, alias="part_client_dbname")
    part_client_instance: str | None = Field(None, alias="part_client_instance")
    part_coll: str | None = Field("part_type", alias="part_coll")
    part_dbconnection: str | None = Field("", alias="part_dbconnection")
    part_stable: bool | None = Field(None, alias="part_stable")
    part_stable_coll: bool | None = Field(False, alias="part_stable_coll")
    part_stable_ordered: bool | None = Field(False, alias="part_stable_ordered")
    part_stable_roundrobin_coll: bool | None = Field(False, alias="part_stable_roundrobin_coll")
    part_table: str | None = Field(None, alias="part_table")
    part_type: WEB_SERVICE.PartType | None = Field(WEB_SERVICE.PartType.auto, alias="part_type")
    part_unique: bool | None = Field(None, alias="part_unique")
    part_unique_coll: bool | None = Field(False, alias="part_unique_coll")
    part_unique_ordered: bool | None = Field(False, alias="part_unique_ordered")
    part_unique_roundrobin_coll: bool | None = Field(False, alias="part_unique_roundrobin_coll")
    perform_sort: bool | None = Field(False, alias="perform_sort")
    perform_sort_coll: bool | None = Field(False, alias="perform_sort_coll")
    perform_sort_modulus: bool | None = Field(False, alias="perform_sort_modulus")
    preserve: WEB_SERVICE.Preserve | None = Field(WEB_SERVICE.Preserve.default_clear, alias="preserve")
    proxy_host_name: str | None = Field("", alias="proxy_host_name")
    proxy_password: str | None = Field("", alias="proxy_password")
    proxy_port_number: str | None = Field("", alias="proxy_port_number")
    proxy_user_name: str | None = Field("", alias="proxy_user_name")
    queue_upper_size: int | None = Field(0, alias="queue_upper_size")
    queue_upper_size_ronly: int | None = Field(0, alias="queue_upper_size_ronly")
    reject_link_index: int | None = Field(-1, alias="rejectLinkIndex")
    reject_link_name: str | None = Field("", alias="rejectLinkName")
    request_mapping: str | None = Field("", alias="request_mapping")
    request_name: str | None = Field("", alias="requestName")
    response_name: str | None = Field("", alias="responseName")
    runtime_column_propagation: int | None = Field(1, alias="runtime_column_propagation")
    show_coll_type: int | None = Field(0, alias="showCollType")
    show_part_type: int | None = Field(1, alias="showPartType")
    show_sort_options: int | None = Field(0, alias="showSortOptions")
    sort_instructions: str | None = Field("", alias="sortInstructions")
    sort_instructions_text: str | None = Field("", alias="sortInstructionsText")
    ssl_certificate: str | None = Field("", alias="ssl_certificate")
    stage_description: list | None = Field("", alias="stageDescription")
    truststore_type: WEB_SERVICE.TruststoreType | None = Field(
        WEB_SERVICE.TruststoreType.JKS, alias="truststore_type"
    )
    user_classpath: str | None = Field("", alias="user_classpath")
    wsdl_document_uri: str | None = Field("", alias="wsdl_document_uri")
    wsdl_document_xml: str | None = Field("", alias="wsdl_document_xml")
    wsdl_input_message_namespace: str | None = Field("", alias="wsdl_input_message_namespace")
    wsdl_operation_name: str | None = Field("", alias="wsdl_operation_name")
    wsdl_operation_style: str | None = Field("", alias="wsdl_operation_style")
    wsdl_port_address: str | None = Field("", alias="wsdl_port_address")
    wsdl_port_name: str | None = Field("", alias="wsdl_port_name")
    wsdl_service_name: str | None = Field("", alias="wsdl_service_name")
    wsdl_soap_action: str | None = Field("", alias="wsdl_soap_action")
    xml_input_column: str | None = Field("", alias="xml_input_column")
    xml_input_header_column: str | None = Field("", alias="xml_input_header_column")
    xml_output_column: str | None = Field("", alias="xml_output_column")
    xml_output_header_column: str | None = Field("", alias="xml_output_header_column")

    def validate_parameters(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        include.add("preserve") if (self.output_count and self.output_count > 0) else exclude.add("preserve")
        (
            include.add("is_log_reject_reason_disabled")
            if (((self.input_count and self.input_count > 0)) and (self.output_count and self.output_count > 0))
            else exclude.add("is_log_reject_reason_disabled")
        )
        include.add("max_mem_buf_size") if (self.buf_mode != "nobuffer") else exclude.add("max_mem_buf_size")
        include.add("buf_free_run") if (self.buf_mode != "nobuffer") else exclude.add("buf_free_run")
        include.add("queue_upper_size") if (self.buf_mode != "nobuffer") else exclude.add("queue_upper_size")
        include.add("disk_write_inc") if (self.buf_mode != "nobuffer") else exclude.add("disk_write_inc")

        (
            include.add("runtime_column_propagation")
            if (not self.enable_schemaless_design)
            else exclude.add("runtime_column_propagation")
        )
        (
            include.add("auto_column_propagation")
            if (not self.output_acp_should_hide)
            else exclude.add("auto_column_propagation")
        )
        (
            include.add("max_mem_buf_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("max_mem_buf_size_ronly")
        )
        (
            include.add("buf_free_run_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("buf_free_run_ronly")
        )
        (
            include.add("queue_upper_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("queue_upper_size_ronly")
        )
        (
            include.add("disk_write_inc_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("disk_write_inc_ronly")
        )
        (
            include.add("part_stable")
            if (
                ((self.show_part_type))
                and (self.part_type != "auto")
                and (self.part_type != "db2connector")
                and (self.show_sort_options)
            )
            else exclude.add("part_stable")
        )
        (
            include.add("part_unique")
            if (
                ((self.show_part_type))
                and (self.part_type != "auto")
                and (self.part_type != "db2connector")
                and (self.show_sort_options)
            )
            else exclude.add("part_unique")
        )
        (
            include.add("key_cols_part")
            if (
                (
                    ((self.show_part_type))
                    and ((not self.show_coll_type))
                    and ((self.part_type != "auto"))
                    and ((self.part_type != "db2connector"))
                    and ((self.part_type != "modulus"))
                )
                or (
                    ((self.show_part_type))
                    and (not self.show_coll_type)
                    and (self.part_type == "modulus")
                    and (self.perform_sort_modulus)
                )
            )
            else exclude.add("key_cols_part")
        )
        (
            include.add("part_dbconnection")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_dbconnection")
        )
        (
            include.add("part_client_dbname")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_client_dbname")
        )
        (
            include.add("part_client_instance")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_client_instance")
        )
        (
            include.add("part_table")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_table")
        )
        (
            include.add("perform_sort")
            if (((self.show_part_type)) and (((self.part_type == "hash")) or (self.part_type == "range")))
            else exclude.add("perform_sort")
        )
        (
            include.add("perform_sort_modulus")
            if (((self.show_part_type)) and (self.part_type == "modulus"))
            else exclude.add("perform_sort_modulus")
        )
        (
            include.add("key_col_select")
            if (
                ((self.show_part_type))
                and (self.part_type == "modulus")
                and (not self.perform_sort_modulus)
            )
            else exclude.add("key_col_select")
        )
        (
            include.add("sort_instructions")
            if (
                ((self.show_part_type))
                and (
                    ((self.part_type == "db2part"))
                    or (self.part_type == "entire")
                    or (self.part_type == "random")
                    or (self.part_type == "roundrobin")
                    or (self.part_type == "same")
                )
            )
            else exclude.add("sort_instructions")
        )
        (
            include.add("sort_instructions_text")
            if (
                ((self.show_part_type))
                and (
                    ((self.part_type == "db2part"))
                    or (self.part_type == "entire")
                    or (self.part_type == "random")
                    or (self.part_type == "roundrobin")
                    or (self.part_type == "same")
                )
            )
            else exclude.add("sort_instructions_text")
        )
        include.add("coll_type") if (self.show_coll_type) else exclude.add("coll_type")
        include.add("part_type") if (self.show_part_type) else exclude.add("part_type")
        (
            include.add("perform_sort_coll")
            if (
                (
                    ((self.show_coll_type))
                    and (
                        ((self.coll_type == "ordered"))
                        or ((self.coll_type == "roundrobin_coll"))
                        or ((self.coll_type == "sortmerge"))
                    )
                )
                or (((not self.show_part_type)) and (not self.show_coll_type))
            )
            else exclude.add("perform_sort_coll")
        )
        (
            include.add("key_cols_coll")
            if (
                ((self.show_coll_type))
                and (not self.show_part_type)
                and (self.coll_type != "auto")
                and (((self.coll_type == "sortmerge")) or (self.perform_sort_coll))
            )
            else exclude.add("key_cols_coll")
        )
        (
            include.add("key_cols_none")
            if (
                ((not self.show_part_type))
                and (not self.show_coll_type)
                and (self.perform_sort_coll)
            )
            else exclude.add("key_cols_none")
        )
        (
            include.add("part_stable_coll")
            if (
                (((self.perform_sort_coll)))
                and (
                    (
                        ((not self.show_part_type))
                        and ((self.show_coll_type))
                        and ((self.coll_type != "auto"))
                        and ((self.show_sort_options))
                    )
                    or (
                        ((not self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.show_sort_options)
                    )
                )
            )
            else exclude.add("part_stable_coll")
        )
        (
            include.add("part_unique_coll")
            if (
                (((self.perform_sort_coll)))
                and (
                    (
                        ((not self.show_part_type))
                        and ((self.show_coll_type))
                        and ((self.coll_type != "auto"))
                        and ((self.show_sort_options))
                    )
                    or (
                        ((not self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.show_sort_options)
                    )
                )
            )
            else exclude.add("part_unique_coll")
        )
        return include, exclude

    def get_input_ports_props(self) -> dict:
        include, exclude = self.validate()
        props = {
            "inherit_from_stage",
            "input_message_table_id",
            "is_input_header_xml_column",
            "is_input_xml_column",
            "link_minimised",
            "log_response",
            "part_coll",
            "runtime_column_propagation",
            "xml_input_column",
            "xml_input_header_column",
        }
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_input_cardinality(self) -> dict:
        return {"min": 0, "max": 1}

    def get_output_ports_props(self) -> dict:
        include, exclude = self.validate()
        props = {
            "input_description",
            "input_name",
            "input_request",
            "is_output_header_xml_column",
            "is_output_xml_column",
            "is_pass_through_disabled",
            "is_reject_column",
            "is_reject_link",
            "output_message_table_id",
            "output_reject_column",
            "request_mapping",
            "xml_output_column",
            "xml_output_header_column",
        }
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_output_cardinality(self) -> dict:
        return {"min": 0, "max": 2}

    def get_parameters_props(self) -> dict:
        include, exclude = self.validate()
        props = {
            "allow_column_mapping",
            "authentication_password",
            "authentication_user_name",
            "auto_column_propagation",
            "buf_free_run",
            "buf_free_run_ronly",
            "buf_mode",
            "buf_mode_ronly",
            "certificate_passwd",
            "coll_type",
            "combinability",
            "current_output_link_type",
            "disk_write_inc",
            "disk_write_inc_ronly",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "error_handling",
            "execmode",
            "field_delimiter",
            "flow_dirty",
            "has_reference_output",
            "hide",
            "inherit_from_stage",
            "input_col",
            "input_count",
            "input_description",
            "input_link_description",
            "input_message_table_id",
            "input_name",
            "input_namespace_name",
            "input_namespace_value",
            "input_request",
            "inputcol_properties",
            "is_authentication_required",
            "is_automatic_trust",
            "is_certificate_required",
            "is_input_header_xml_column",
            "is_input_xml_column",
            "is_log_reject_reason_disabled",
            "is_output_header_xml_column",
            "is_output_xml_column",
            "is_pass_through_disabled",
            "is_proxy_required",
            "is_reject_column",
            "is_reject_link",
            "jvm_options",
            "key_col_select",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "keystore_file",
            "link_minimised",
            "log_response",
            "max_mem_buf_size",
            "max_mem_buf_size_ronly",
            "operation_timeout",
            "output_acp_should_hide",
            "output_col",
            "output_count",
            "output_link_description",
            "output_message_table_id",
            "output_namespace_name",
            "output_namespace_value",
            "output_reject_column",
            "outputcol_properties",
            "part_client_dbname",
            "part_client_instance",
            "part_coll",
            "part_dbconnection",
            "part_stable",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_table",
            "part_type",
            "part_unique",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "preserve",
            "proxy_host_name",
            "proxy_password",
            "proxy_port_number",
            "proxy_user_name",
            "queue_upper_size",
            "queue_upper_size_ronly",
            "reject_link_index",
            "reject_link_name",
            "request_mapping",
            "request_name",
            "response_name",
            "runtime_column_propagation",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "ssl_certificate",
            "stage_description",
            "truststore_type",
            "user_classpath",
            "wsdl_document_uri",
            "wsdl_document_xml",
            "wsdl_input_message_namespace",
            "wsdl_operation_name",
            "wsdl_operation_style",
            "wsdl_port_address",
            "wsdl_port_name",
            "wsdl_service_name",
            "wsdl_soap_action",
            "xml_input_column",
            "xml_input_header_column",
            "xml_output_column",
            "xml_output_header_column",
        }
        required = {
            "current_output_link_type",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "output_acp_should_hide",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_app_data_props(self) -> dict:
        return {
            "datastage": {
                "maxRejectOutputs": 1,
                "minRejectOutputs": 0,
                "maxReferenceInputs": 0,
                "minReferenceInputs": 0,
            }
        }



import ibm_watsonx_data_integration.services.datastage.models.flow_json_model as model
from enum import Enum
from pydantic import BaseModel, ConfigDict, Field, computed_field, field_serializer
from typing import Any, Literal


class NativeType(Enum):
    Binary = "BINARY"
    NativeBinary = "NATIVE_BINARY"
    Decimal = "DECIMAL"
    DisplayNumeric = "DISPLAY_NUMERIC"
    Float = "FLOAT"
    Character = "CHARACTER"
    VarChar = "VARCHAR"
    Group = "GROUP"
    GraphicN = "GRAPHIC_N"
    GraphicG = "GRAPHIC_G"
    VarGraphicN = "VARGRAPHIC_N"
    VarGraphicG = "VARGRAPHIC_G"


class Usage(Enum):
    none = ""
    COMP = "COMP"
    COMP_1 = "COMP-1"
    COMP_2 = "COMP-2"
    COMP_3 = "COMP-3"
    COMP_5 = "COMP-5"
    DISPLAY = "DISPLAY"
    DISPLAY_1 = "DISPLAY-1"


class Extended(Enum):
    Unsigned = "unsigned"
    Unicode = "unicode"


class SignOption(Enum):
    none = 0
    Leading = "1"
    Trailing = "2"
    LeadingSeparate = "3"
    TrailingSeparate = "4"


class ArrayHandling(Enum):
    ArrayHandling = 0
    ArrayHandlingAsIs = 1
    ArrayHandlingDenormalize = 2


class Column(BaseModel):
    model_config = ConfigDict(populate_by_name=True)

    name: str = Field(None, alias="name")
    native_type: NativeType = Field(NativeType.Character, alias="native_type")
    length: int = Field(None, alias="max_length")
    min_length: int = Field(0, alias="min_length")
    scale: int = Field(None, alias="decimal_scale")
    level: int = Field(2, alias="item_index")
    is_unicode_string: bool = Field(False, alias="is_unicode_string")
    is_signed: bool = Field(False, alias="is_signed")
    usage: Usage = Field(Usage.none, alias="usage")
    description: str = Field(None, alias="description")
    nullable: bool = Field(False, alias="nullable")
    sign_option: SignOption = Field(SignOption.none, alias="sign_option")
    sync_indicator: bool = Field(False, alias="sync_indicator")
    has_sign_indicator: bool = Field(True, alias="has_sign_indicator")
    default: Any = Field(None, alias="default")
    print_fields: bool = Field(False, alias="print_field")
    field_max_width: int = Field(None, alias="max_width")
    field_width: int = Field(None, alias="width")
    prefix_bytes: Literal[0, 1, 2, 4] = Field(None, alias="prefix")
    occurs: int = Field(None, alias="dimension_max_size")
    redefined_field: str = Field(None, alias="redefined_field")
    depending_on: str = Field(None, alias="depend_field")
    # make this a literal?
    date_format: str = Field(None, alias="date_mask")
    mf_update_value: str = Field(None, alias="mf_update_value")
    array_handling: ArrayHandling = Field(ArrayHandling.ArrayHandling, alias="array_handling")

    @field_serializer("length")
    def serialize_length(self, length: int | None) -> int:
        if length:
            return length
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        if native_type in ["BINARY"]:
            return 18
        elif native_type in ["NATIVE_BINARY", "DECIMAL", "DISPLAY_NUMERIC", "FLOAT"]:
            return 31
        else:
            return 100

    @field_serializer("usage")
    def serialize_usage(self, usage: Usage | None) -> Usage | str:
        if usage:
            return usage
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        if native_type == "BINARY":
            return Usage.COMP
        elif native_type == "NATIVE_BINARY":
            return Usage.COMP_5
        elif native_type == "DECIMAL":
            return Usage.COMP_3
        elif native_type == "FLOAT":
            return Usage.COMP_2
        elif native_type in ["GRAPHIC_G", "VARGRAPHIC_G"]:
            return Usage.DISPLAY_1
        elif native_type in ["GRAPHIC_N", "VARGRAPHIC_N"]:
            return ""
        else:
            return Usage.DISPLAY

    @field_serializer("occurs")
    def serialize_occurs(self, occurs: int) -> str:
        return str(occurs)

    @computed_field
    @property
    def decimal_precision(self) -> int:
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        if native_type in ["NATIVE_BINARY", "DECIMAL", "DISPLAY_NUMERIC", "FLOAT"]:
            return self.length
        return 0

    @computed_field
    @property
    def dimension_min_size(self) -> str | None:
        if self.occurs:
            return str(self.occurs)
        return None

    # apt field properties persists throughout all columns in the record
    @computed_field
    @property
    def apt_field_properties(self) -> str:
        apt_string = []
        if self.prefix_bytes:
            apt_string.append(f"prefix={self.prefix_bytes}")
        if self.default:
            apt_string.append(f"default={self.default}")
        if self.field_max_width:
            apt_string.append(f"max_width={self.field_max_width}")
        if self.field_width:
            apt_string.append(f"width={self.field_width}")
        if self.print_fields:
            apt_string.append("print_field")
        return ", ".join(apt_string)

    @computed_field
    @property
    def odbc_type(self) -> str:
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        native_to_odbc = {
            "BINARY": "INTEGER",
            "NATIVE_BINARY": "DECIMAL",
            "DECIMAL": "DECIMAL",
            "DISPLAY_NUMERIC": "DECIMAL",
            "FLOAT": "DECIMAL",
            "VARCHAR": "VARCHAR",
            "GROUP": "CHAR",
            "GRAPHIC_N": "WCHAR",
            "GRAPHIC_G": "WCHAR",
            "VARGRAPHIC_N": "WVARCHAR",
            "VARGRAPHIC_G": "WVARCHAR",
        }
        if native_type in native_to_odbc:
            return native_to_odbc[native_type]
        if native_type == "CHARACTER":
            if self.date_format:
                return "DATE"
            else:
                return "CHAR"
        # raise error?
        return "CHAR"

    @computed_field
    @property
    def type_code(self) -> str:
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        native_to_type_code = {
            "BINARY": "INT32",
            "NATIVE_BINARY": "DECIMAL",
            "DECIMAL": "DECIMAL",
            "DISPLAY_NUMERIC": "DECIMAL",
            "FLOAT": "DECIMAL",
            "VARCHAR": "STRING",
            "GROUP": "STRING",
            "GRAPHIC_N": "STRING",
            "GRAPHIC_G": "STRING",
            "VARGRAPHIC_N": "STRING",
            "VARGRAPHIC_G": "STRING",
        }
        if native_type in native_to_type_code:
            return native_to_type_code[native_type]
        if native_type == "CHARACTER":
            if self.date_format:
                return "DATE"
            else:
                return "STRING"
        return "STRING"

    @computed_field
    @property
    def type(self) -> str:
        native_type = self.native_type.value if isinstance(self.native_type, Enum) else self.native_type
        native_to_type = {
            "BINARY": "integer",
            "NATIVE_BINARY": "double",
            "DECIMAL": "double",
            "DISPLAY_NUMERIC": "double",
            "FLOAT": "double",
            "VARCHAR": "string",
            "GROUP": "string",
            "GRAPHIC_N": "string",
            "GRAPHIC_G": "string",
            "VARGRAPHIC_N": "string",
            "VARGRAPHIC_G": "string",
        }
        if native_type in native_to_type:
            return native_to_type[native_type]
        if native_type == "CHARACTER":
            if self.date_format:
                return "date"
            else:
                return "string"
        return "string"

    def _to_field(self) -> model.FieldModel:
        metadata_props = {
            "length",
            "min_length",
            "decimal_precision",
            "scale",
            "is_signed",
            "level",
            "description",
        }
        app_data_props = {
            "usage",
            "sign_option",
            "sync_indicator",
            "is_unicode_string",
            "odbc_type",
            "type_code",
            "native_type",
            "has_sign_indicator",
            # "mf_update_value",
            "occurs",
            "dimension_min_size",
            "depending_on",
            "redefined_field",
            "date_format",
            "apt_field_properties",
        }

        return model.FieldModel(
            name=self.name,
            type=self.type,
            nullable=self.nullable,
            metadata=self.model_dump(include=metadata_props, exclude_none=True, by_alias=True),
            app_data=self.model_dump(include=app_data_props, exclude_none=True, by_alias=True),
        )

    def _to_output_field(self) -> model.FieldModel:
        metadata_props = {
            "length",
            "min_length",
            "decimal_precision",
            "scale",
            "is_signed",
            "level",
            "description",
        }
        app_data_props = {
            "usage",
            "sign_option",
            "sync_indicator",
            "is_unicode_string",
            "odbc_type",
            "type_code",
            "native_type",
            "has_sign_indicator",
            "occurs",
            "dimension_min_size",
            "depending_on",
            "redefined_field",
            "date_format",
        }

        return model.FieldModel(
            name=self.name,
            type=self.type,
            nullable=self.nullable,
            metadata=self.model_dump(include=metadata_props, exclude_none=True, by_alias=True),
            app_data=self.model_dump(include=app_data_props, exclude_none=True, by_alias=True),
        )

    @classmethod
    def from_dict(cls, dict: dict):
        if "apt_field_properties" in dict:
            apt_props = dict["apt_field_properties"].split(", ")
            for prop in apt_props:
                if "=" in prop:
                    key_val = prop.split("=")
                    assert len(key_val) == 2
                    key, val = key_val[0].strip(), key_val[1].strip()
                    dict[key] = val
                    if key in ["prefix", "max_width", "width"]:
                        dict[key] = int(val)
                    else:
                        dict[key] = val
                elif "print_field" in prop:
                    dict["print_field"] = True
        return cls(**dict)


class Record(BaseModel):
    name: str = Field(None, alias="name")
    columns: list[Column] = Field([], alias="columns")


class RecordID(BaseModel):
    model_config = ConfigDict(populate_by_name=True)

    record_name: str = Field(None, alias="record_name")
    column_name: str = Field(None, alias="record_id_name")
    operator: str = Field("=", alias="record_id_name_value_relation")
    value: str = Field(None, alias="record_id_value")
    schema_ref: str = Field(None, alias="schema_ref")

    @computed_field
    @property
    def name(self) -> str:
        return self.record_name


class OutputColumns(BaseModel):
    output_name: str = Field(None, alias="output_name")
    output_columns: list = Field([], alias="output_columns")


class Constraint(BaseModel):
    output_name: str = Field(None, alias="output_name")
    constraint: str = Field([], alias="constraint")


class complex_flat_file:
    Column = Column
    Record = Record
    RecordID = RecordID
    OutputColumns = OutputColumns
    Constraint = Constraint
    NativeType = NativeType
    Usage = Usage
    Extended = Extended
    SignOption = SignOption
    ArrayHandling = ArrayHandling



from ibm_watsonx_data_integration.services.datastage.models.enums import AZURE_COSMOS_CONNECTION
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestAzureCosmosConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "azurecosmos_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _aaa_tab_test_1_ = fc.add_stage("Microsoft Azure Cosmos DB", "aaa_tab_test_1")
        _aaa_tab_test_1_.configuration.collection = "aaa_tab_test"
        _aaa_tab_test_1_.configuration.database = "tm_cc_db_1"

        _aaa_tab_test_1_.configuration.connection.authentication_method = AZURE_COSMOS_CONNECTION.AuthMethod.master_key
        _aaa_tab_test_1_.configuration.connection.hostname = environ["AZURECOSMOS_HOST"]
        _aaa_tab_test_1_.configuration.connection.master_key = environ["AZURECOSMOS_MASTER_KEY"]

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        # Graph
        aaa_tab_test_1_schema = _aaa_tab_test_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        aaa_tab_test_1_schema.add_field("BIGINT", "col_int").nullable()
        aaa_tab_test_1_schema.add_field("VARCHAR", "col_varchar").nullable().length(1024)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025


"""Module containing Roles Models."""

import json
import requests
from enum import Enum
from ibm_watsonx_data_integration.common.models import BaseModel, CollectionModel, CollectionModelResults
from pydantic import Field
from typing import TYPE_CHECKING, Any, Optional

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.platform import Platform


class RoleType(str, Enum):
    """The model for RoleType enums."""

    SYSTEM_ROLE = "system_role"
    SERVICE_ROLE = "service_role"
    CUSTOM_ROLE = "custom_role"


class Role(BaseModel):
    """The model for a role."""

    display_name: str = Field(repr=True)
    role_type: RoleType = Field(repr=True, frozen=True)
    actions: list[str] = Field(repr=True)
    description: str | None = Field("", repr=False)
    crn: str | None = Field("", frozen=True, repr=False)
    name: str = Field(None, repr=False, frozen=True)
    account_id: str = Field(None, frozen=True, repr=False)
    service_name: str = Field(None, repr=False, frozen=True)
    id: str = Field("", frozen=True, repr=False)
    created_at: str | None = Field("", frozen=True, repr=False)
    created_by_id: str | None = Field("", frozen=True, repr=False)
    last_modified_at: str | None = Field("", frozen=True, repr=False)
    last_modified_by_id: str | None = Field("", frozen=True, repr=False)
    href: str | None = Field("", frozen=True, repr=False)
    etag: str | None = Field("", repr=False)

    def __init__(self, platform: Optional["Platform"] = None, **custom_role_json: dict) -> None:
        """The __init__ of the CustomRole class.

        Args:
            platform: The Platform object.
            custom_role_json: The JSON for the CustomRole.
        """
        super().__init__(**custom_role_json)
        self._platform = platform

    @staticmethod
    def _create(
        platform: "Platform",
        name: str,
        display_name: str,
        service_name: str,
        actions: list,
        description: str | None = None,
    ) -> "Role":
        if name and name[0].islower():
            name = name[0].upper() + name[1:]

        data = {
            "name": name,
            "display_name": display_name,
            "account_id": platform.current_account.account_id,
            "service_name": service_name,
            "actions": actions,
        }

        if description:
            data["description"] = description

        response = platform._role_api_client.create_role(json.dumps(data))
        response_json = response.json()
        response_json["role_type"] = RoleType.CUSTOM_ROLE
        return Role(platform=platform, **response_json)

    def _update(self) -> requests.Response:
        if self.role_type is not RoleType.CUSTOM_ROLE.value:
            raise TypeError("You can only update a custom role.")

        data = {
            "display_name": self.display_name,
            "description": self.description,
            "actions": self.actions,
        }

        if not self.etag:
            api_response = self._platform._role_api_client.retrieve_role(self.id)
            etag = api_response.headers.get("etag")
        else:
            etag = self.etag

        return self._platform._role_api_client.update_role(self.id, etag, json.dumps(data))

    def _delete(self) -> requests.Response:
        if self.role_type is not RoleType.CUSTOM_ROLE.value:
            raise TypeError("You can only delete a custom role.")

        return self._platform._role_api_client.delete_role(self.id)


class Roles(CollectionModel):
    """Collection of Roles instances."""

    def __init__(self, platform: "Platform") -> None:
        """The __init__ of the Role class.

        Args:
            platform: The Platform object.
        """
        super().__init__(platform=platform)
        self.unique_id = "display_name"

    def __len__(self) -> int:
        """Total amount of roles."""
        request_params_defaults = {
            "account_id": self._platform.current_account.account_id,
        }

        api_response = self._platform._role_api_client.get_roles(request_params_defaults).json()
        length = (
            len(api_response["system_roles"]) + len(api_response["service_roles"]) + len(api_response["custom_roles"])
        )
        return length

    def _request_parameters(self) -> list:
        return ["role_id", "account_id", "service_name", "source_service_name", "policy_type", "service_group_id"]

    def _get_results_from_api(self, request_params: dict = None, **kwargs: dict) -> CollectionModelResults:
        """Returns results of an api request."""
        request_params_defaults = {
            "role_id": None,
            "account_id": self._platform.current_account.account_id,
            "service_name": None,
            "source_service_name": None,
            "policy_type": None,
            "service_group_id": None,
        }
        request_params_unioned: dict[str, Any] = request_params_defaults
        if request_params:
            request_params_unioned.update(request_params)

        role_id = request_params_unioned.get("role_id")

        if role_id:
            api_response = self._platform._role_api_client.retrieve_role(role_id)
            etag = api_response.headers.get("etag")
            response_json = api_response.json()
            response_json["etag"] = etag
            response_json["role_type"] = "custom_role"
            response = {"results": [response_json]}
        else:
            api_response = self._platform._role_api_client.get_roles(
                params={k: v for k, v in request_params_unioned.items() if v is not None}
            ).json()

            response = {
                "results": [
                    {**role, "role_type": role_type}
                    for key, role_type in [
                        ("system_roles", RoleType.SYSTEM_ROLE),
                        ("service_roles", RoleType.SERVICE_ROLE),
                        ("custom_roles", RoleType.CUSTOM_ROLE),
                    ]
                    for role in api_response.get(key, [])
                ]
            }

        return CollectionModelResults(
            response,
            Role,
            "",
            "",
            "results",
            constructor_params={"platform": self._platform},
        )



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""Module contains base class for all API Clients."""

import requests
import urllib3
from abc import ABC
from dataclasses import dataclass
from ibm_watsonx_data_integration.common.auth import BaseAuthenticator
from ibm_watsonx_data_integration.common.exceptions import IbmCloudApiException
from ibm_watsonx_data_integration.cpd_api.adapters import DefaultHTTPAdapter
from requests.exceptions import HTTPError
from typing import Any


@dataclass
class BaseAPIClient(ABC):
    """Base Abstract API Client class.

    Every APIClient class should Inherit from this one.
    """

    base_url: urllib3.util.Url
    _auth: BaseAuthenticator
    _session: requests.Session

    def __init__(self, base_url: str, auth: BaseAuthenticator) -> None:
        """Initialize API Client classes.

        To customize behaviour per host in all API Clients, add adapter for this host below.

        Args:
            base_url: The Cloud Pak for Data URL.
            auth: The Authentication object.
        """
        # strip trailing "/" so that we do not end up with "//" when appending paths to urls.
        base_url = base_url.rstrip("/")
        self.base_url = urllib3.util.parse_url(base_url)
        self._auth = auth
        self._session = requests.Session()

        retries = urllib3.Retry(
            total=3,
            backoff_factor=1,
            status_forcelist=[429],
            raise_on_status=True,
            allowed_methods={"GET", "POST", "PUT", "PATCH", "DELETE"},
        )

        self._default_adapter = DefaultHTTPAdapter(auth=self._auth, max_retries=retries)
        self._session.mount("http://", self._default_adapter)
        self._session.mount("https://", self._default_adapter)

    def _request(
        self,
        method: str,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP methods.

        Main entrypoint that call all requests.

        Args:
            method: HTTP method name (GET, POST, ...).
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over header's parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        if adapter is not None:
            adapter._auth = self._auth
            self._session.mount(url, adapter)
        elif headers:
            self._default_adapter._custom_headers = headers

        _url = urllib3.util.parse_url(url)
        try:
            response = self._session.request(
                method=method,
                url=urllib3.util.Url(
                    scheme=self.base_url.scheme,
                    host=self.base_url.host,
                    port=self.base_url.port,
                    path=_url.path,
                ),
                params=params,
                data=data,
                verify=verify,
            )
            try:
                # raise HTTPError on HTTP error codes in range [400; 600)
                response.raise_for_status()
            except HTTPError as exc:
                raise IbmCloudApiException(response=exc.response) from exc

        finally:
            if adapter is not None:
                adapter._auth = None
                self._session.adapters.pop(url)
            elif headers:
                self._default_adapter._custom_headers = {}
        return response

    def get(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP GET method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(method="GET", url=url, params=params, adapter=adapter, headers=headers, verify=verify)

    def post(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP POST method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(
            method="POST", url=url, params=params, data=data, adapter=adapter, headers=headers, verify=verify
        )

    def put(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP PUT method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(
            method="PUT", url=url, params=params, data=data, adapter=adapter, headers=headers, verify=verify
        )

    def patch(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP PATCH method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(
            method="PATCH", url=url, params=params, data=data, adapter=adapter, headers=headers, verify=verify
        )

    def delete(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP DELETE method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(
            method="DELETE", url=url, params=params, data=data, adapter=adapter, headers=headers, verify=verify
        )

    def head(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        data: dict | str | None = None,
        adapter: DefaultHTTPAdapter | None = None,
        headers: dict | None = None,
        verify: bool | str = True,
    ) -> requests.Response:
        """Proxy method for all HTTP HEAD method calls.

        Args:
            url: URL to requests for
            params: HTTP request parameters.
            data: HTTP request payload.
            adapter: HTTP request adapter. Has priority over headers parameter.
            headers: HTTP request headers to add to the request. Can overwrite default set headers.
            verify: Whether to verify a request. Can also be a path to a certificate file.

        Returns:
            A HTTP response.
        """
        return self._request(
            method="HEAD", url=url, params=params, data=data, adapter=adapter, headers=headers, verify=verify
        )



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025


"""Module containing Account Model."""

from ibm_watsonx_data_integration.common.models import BaseModel, CollectionModel, CollectionModelResults
from pydantic import ConfigDict, Field, PrivateAttr
from typing import TYPE_CHECKING, Any, ClassVar, Optional

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.platform import Platform


class AccountMetadata(BaseModel):
    """Model representing metadata for an account."""

    # required and is hidden from the model’s string representation.
    guid: str = Field(repr=False)
    url: str = Field(repr=False)
    created_at: str = Field(repr=False)
    created_by: str | None = Field(default=None, repr=False)
    updated_at: str = Field(repr=False)
    updated_by: str = Field(repr=False)
    update_comments: str = Field(repr=False)

    model_config = ConfigDict(frozen=True)
    _expose: bool = PrivateAttr(default=False)


class BluemixSubscription(BaseModel):
    """Model representing a Bluemix subscription with payment details."""

    type: str = Field(repr=False)
    state: str = Field(repr=False)
    payment_method: dict = Field(repr=False)
    subscription_id: str = Field(repr=False)
    part_number: str = Field(repr=False)
    subscriptionTags: list[Any] = Field(repr=False)
    history: list[Any] | None = Field(default=None, repr=False)
    current_state_timestamp: str = Field(repr=False)
    billing_system: str = Field(repr=False)
    category: str = Field(repr=False)

    model_config = ConfigDict(frozen=True)
    _expose: bool = PrivateAttr(default=False)

    @property
    def payment_type(self) -> str | None:
        """Access the ``type`` in payment_method.

        Returns:
            A payment type.
        """
        return self.payment_method.get("type")

    @property
    def payment_started(self) -> str | None:
        """Access the ``started`` in payment_method.

        Returns:
            A started payment type.
        """
        return self.payment_method.get("started")

    @property
    def payment_ended(self) -> str | None:
        """Access the ``ended`` in payment_method.

        Returns:
            A ended payment type.
        """
        return self.payment_method.get("ended")


class Account(BaseModel):
    """Model representing an account, including metadata and flattened entity details."""

    metadata: AccountMetadata = Field(repr=False)

    name: str = Field(repr=True)
    account_type: str = Field(alias="type", repr=True)
    state: str = Field(repr=False)
    # not required, default to an empty string ("") and is hidden from the model’s string representation.
    owner: str | None = Field("", repr=False)
    owner_userid: str = Field(repr=False)
    owner_unique_id: str = Field(repr=False)
    iam_id: str = Field(alias="owner_iam_id", repr=False)
    account_id: str = Field(alias="customer_id", repr=True, default_factory=lambda fields: fields["metadata"].guid)
    country_code: str = Field(repr=False)
    currency_code: str = Field(repr=False)
    billing_country_code: str = Field(repr=False)
    isIBMer: bool = Field(repr=False)
    terms_and_conditions: dict = Field(repr=False)
    tags: list[Any] = Field(default_factory=list, repr=False)
    team_directory_enabled: bool = Field(repr=False)
    organizations_region: list[Any] = Field(default_factory=list, repr=False)
    linkages: list[dict] = Field(repr=False)
    bluemix_subscriptions: list[BluemixSubscription] = Field(repr=False)
    subscription_id: str = Field(repr=False)
    current_billing_system: str = Field(repr=False)
    configuration_id: str | None = Field(default=None, repr=False)
    onboarded: int = Field(repr=False)
    origin: str = Field(repr=False)

    model_config = ConfigDict(frozen=True)
    _expose: bool = True
    EXPOSED_DATA_PATH: ClassVar[dict] = {"entity": {}}

    def __init__(self, platform: Optional["Platform"] = None, **account_json: dict) -> None:
        """The __init__ of the Account.

        Args:
            platform: The Platform object.
            account_json: The JSON for the Account.
        """
        super().__init__(**account_json)
        self._platform = platform


class Accounts(CollectionModel):
    """Collection of Account instances."""

    def __init__(self, platform: "Platform") -> None:
        """The __init__ of the Accounts class.

        Args:
            platform: The Platform object.
        """
        super().__init__(platform)
        self.unique_id = "metadata.guid"

    def __len__(self) -> int:
        """Total number of accounts."""
        query_params = {
            "limit": 1,
        }
        res = self._platform._account_api.get_accounts(params=query_params)
        res_json = res.json()
        return res_json["total_results"]

    def _request_parameters(self) -> list:
        return ["account_id"]

    def _get_results_from_api(self, request_params: dict = None, **kwargs: dict) -> CollectionModelResults:
        """Returns results of an api request."""
        # TODO (WSDK-196): Add filtering parameters once they have been verified to work correctly.

        request_params_defaults = {
            "account_id": None,
        }
        request_params_unioned = request_params_defaults.copy()
        request_params_unioned.update(request_params)
        account_id = request_params_unioned.get("account_id")

        if account_id:
            response = self._platform._account_api.get_account(account_id).json()
            response = {"resources": [response]}
        else:
            response = self._platform._account_api.get_accounts(
                params={k: v for k, v in request_params_unioned.items() if v is not None}
            ).json()

        if "next_url" in response and response["next_url"] is None:
            response.pop("next_url")

        return CollectionModelResults(
            response,
            Account,
            "next_url",
            "next_url",
            "resources",
            {"platform": self._platform},
        )



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import GENERICS3_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class ListObjectsApiVersion(Enum):
    v1 = "v1"
    v2 = "v2"


class Generics3Conn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "38714ac2-8f66-4a8c-9b40-806ffb61c759"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    access_key: str = Field(None, alias="access_key")
    bucket: str | None = Field(None, alias="bucket")
    das_api_key: str = Field(None, alias="cas_api_key")
    das_endpoint: str = Field(None, alias="cas_endpoint")
    das_instance_id: str = Field(None, alias="cas_instance_id")
    catalog_name: str | None = Field(None, alias="catalog_name")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    disable_chunked_encoding: bool | None = Field(False, alias="disable_chunked_encoding")
    enable_global_bucket_access: bool | None = Field(True, alias="enable_global_bucket_access")
    enable_path_style_access: bool | None = Field(None, alias="enable_path_style_access")
    s3_list_objects_version: GENERICS3_CONNECTION.ListObjectsApiVersion | None = Field(
        GENERICS3_CONNECTION.ListObjectsApiVersion.v1, alias="list_objects_api_version"
    )
    iceberg_rest_catalog_endpoint: str | None = Field(None, alias="mds_rest_endpoint")
    region: str | None = Field(None, alias="region")
    secret_key: str = Field(None, alias="secret_key")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    validate_ssl_certificate: bool | None = Field(None, alias="ssl_certificate_validation")
    trust_all_ssl_certificates: bool | None = Field(False, alias="trust_all_ssl_cert")
    url: str = Field(None, alias="url")
    use_watsonx_credential_provider: bool | None = Field(None, alias="use_watsonx_credential_provider")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        (
            include.add("secret_key")
            if (((not self.defer_credentials)) and (not self.use_watsonx_credential_provider))
            else exclude.add("secret_key")
        )
        include.add("das_api_key") if (self.use_watsonx_credential_provider) else exclude.add("das_api_key")
        (
            include.add("access_key")
            if (((not self.defer_credentials)) and (not self.use_watsonx_credential_provider))
            else exclude.add("access_key")
        )
        include.add("das_instance_id") if (self.use_watsonx_credential_provider) else exclude.add("das_instance_id")
        include.add("das_endpoint") if (self.use_watsonx_credential_provider) else exclude.add("das_endpoint")
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        include.add("das_api_key") if (self.use_watsonx_credential_provider) else exclude.add("das_api_key")
        (
            include.add("secret_key")
            if (not self.defer_credentials) and (not self.use_watsonx_credential_provider)
            else exclude.add("secret_key")
        )
        include.add("das_endpoint") if (self.use_watsonx_credential_provider) else exclude.add("das_endpoint")
        include.add("ssl_certificate_file") if (not self.ssl_certificate) else exclude.add("ssl_certificate_file")
        include.add("das_instance_id") if (self.use_watsonx_credential_provider) else exclude.add("das_instance_id")
        include.add("ssl_certificate") if (not self.ssl_certificate_file) else exclude.add("ssl_certificate")
        (
            include.add("access_key")
            if (not self.defer_credentials) and (not self.use_watsonx_credential_provider)
            else exclude.add("access_key")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from ibm_watsonx_data_integration.services.datastage.models.enums import CLOUD_OBJECT_STORAGE
from os import environ
from tests.integration.datastage.util import AbstractFlowRunTest
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestCloudObjectStorageConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "cloudobjectstorage_peek"

    def get_fc(self, project: Project):
        fc = project.create_flow(name=self.get_flow_name(), environment=None, description="", flow_type="datastage")

        # Stages
        _abccsv_1_ = fc.add_stage("IBM Cloud Object Storage", "abccsv_1")
        _abccsv_1_.configuration.escape_character = CLOUD_OBJECT_STORAGE.EscapeCharacter.double_quote
        _abccsv_1_.configuration.file_name = "abc.csv"
        _abccsv_1_.configuration.infer_as_varchar = False
        _abccsv_1_.configuration.infer_schema = False
        _abccsv_1_.configuration.quote_character = CLOUD_OBJECT_STORAGE.QuoteCharacter.double_quote

        _abccsv_1_.configuration.connection.access_key = environ["CLOUDOBJECTSTORAGE_ACCESS_KEY"]
        _abccsv_1_.configuration.connection.authentication_method = "accesskey_secretkey"
        _abccsv_1_.configuration.connection.iam_url = "https://iam.cloud.ibm.com/identity/token"
        _abccsv_1_.configuration.connection.secret_key = environ["CLOUDOBJECTSTORAGE_SECRET_KEY"]
        _abccsv_1_.configuration.connection.trust_all_ssl_certificates = False
        _abccsv_1_.configuration.connection.url = environ["CLOUDOBJECTSTORAGE_URL"]
        _abccsv_1_.configuration.connection.bucket = "ds-fastpath-test-bucket"

        _peek_1_ = fc.add_stage("Peek", "Peek_1")

        abccsv_1_schema = _abccsv_1_.connect_output_to(_peek_1_).set_name("Link_1").create_schema()

        # Schemas
        abccsv_1_schema.add_field("VARCHAR", "COLUMN1").nullable().length(1024)
        abccsv_1_schema.add_field("VARCHAR", "COLUMN2").nullable().length(1024)
        abccsv_1_schema.add_field("VARCHAR", "COLUMN3").nullable().length(1024)
        abccsv_1_schema.add_field("DOUBLE", "COLUMN4").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN5").nullable()
        abccsv_1_schema.add_field("BIGINT", "COLUMN6").nullable()
        abccsv_1_schema.add_field("DOUBLE", "COLUMN7").nullable()
        abccsv_1_schema.add_field("BIT", "COLUMN8").nullable()
        abccsv_1_schema.add_field("DATE", "COLUMN9").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN10").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN11").nullable()
        abccsv_1_schema.add_field("VARCHAR", "COLUMN12").nullable().length(1024)
        abccsv_1_schema.add_field("BIGINT", "COLUMN13").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN14").nullable()
        abccsv_1_schema.add_field("VARCHAR", "COLUMN15").nullable().length(1024)
        abccsv_1_schema.add_field("INTEGER", "COLUMN16").nullable()
        abccsv_1_schema.add_field("DOUBLE", "COLUMN17").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN18").nullable()
        abccsv_1_schema.add_field("TIME", "COLUMN19").nullable()
        abccsv_1_schema.add_field("TIMESTAMP", "COLUMN20").nullable()
        abccsv_1_schema.add_field("INTEGER", "COLUMN21").nullable()
        abccsv_1_schema.add_field("BIGINT", "COLUMN22").nullable()
        abccsv_1_schema.add_field("VARCHAR", "COLUMN23").nullable().length(1024)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import Literal

StageTypeStr = Literal[
    "Address Verification",
    "Aggregator",
    "Amazon RDS for MySQL",
    "Amazon RDS for Oracle",
    "Amazon RDS for PostgreSQL",
    "Amazon Redshift",
    "Amazon S3",
    "Apache Cassandra",
    "Apache Cassandra for DataStage",
    "Apache Derby",
    "Apache HBase",
    "Apache HDFS",
    "Apache Hive",
    "Apache Impala",
    "Apache Kafka",
    "Azure PostgreSQL",
    "Big Data File",
    "Bloom Filter",
    "Box",
    "Change Apply",
    "Change Capture",
    "Checksum",
    "Column Export",
    "Column Generator",
    "Column Import",
    "Combine Records",
    "Compare",
    "Complex Flat File",
    "Compress",
    "Copy",
    "Data Rules",
    "Data Service",
    "Data set",
    "DataStax Enterprise",
    "Decode",
    "Denodo",
    "Difference",
    "Distributed Transaction",
    "Dremio",
    "Dropbox",
    "Elasticsearch",
    "Encode",
    "Exasol",
    "Excel",
    "Expand",
    "External Filter",
    "External Source",
    "External Target",
    "FTP",
    "File set",
    "Filter",
    "Funnel",
    "Generic",
    "Generic JDBC",
    "Generic S3",
    "Google BigQuery",
    "Google Cloud Pub/Sub",
    "Google Cloud Storage",
    "Google Looker",
    "Greenplum",
    "HTTP",
    "Head",
    "Hierarchical Data",
    "IBM Cloud Data Engine",
    "IBM Cloud Databases for DataStax",
    "IBM Cloud Databases for MongoDB",
    "IBM Cloud Databases for MySQL",
    "IBM Cloud Databases for PostgreSQL",
    "IBM Cloud Object Storage",
    "IBM Cognos Analytics",
    "IBM Data Virtualization",
    "IBM Data Virtualization Manager for z/OS",
    "IBM Db2",
    "IBM Db2 Big SQL",
    "IBM Db2 Event Store",
    "IBM Db2 Hosted",
    "IBM Db2 Warehouse",
    "IBM Db2 for DataStage",
    "IBM Db2 for i",
    "IBM Db2 for z/OS",
    "IBM Db2 on Cloud",
    "IBM Informix",
    "IBM MQ",
    "IBM Match 360",
    "IBM Netezza Performance Server",
    "IBM Netezza Performance Server for DataStage",
    "IBM Planning Analytics",
    "IBM Product Master",
    "IBM watsonx.data Presto",
    "Investigate",
    "Java Integration",
    "Join",
    "Lookup",
    "Lookup file set",
    "Make Subrecord",
    "Make Vector",
    "MariaDB",
    "Match Frequency",
    "MatchFreq",
    "Merge",
    "Microsoft Azure Blob Storage",
    "Microsoft Azure Cosmos DB",
    "Microsoft Azure Data Lake Storage",
    "Microsoft Azure Databricks",
    "Microsoft Azure File Storage",
    "Microsoft Azure SQL Database",
    "Microsoft Azure Synapse Analytics",
    "Microsoft SQL Server",
    "MinIO",
    "Modify",
    "MongoDB",
    "MySQL",
    "ODBC",
    "One-source Match",
    "Operational Decision Manager",
    "Oracle",
    "Oracle Database for DataStage",
    "Peek",
    "Pivot Enterprise",
    "PostgreSQL",
    "Presto",
    "Promote Subrecord",
    "QSmwi",
    "Remove Duplicates",
    "Rest",
    "Row Generator",
    "SAP ASE",
    "SAP BAPI",
    "SAP Bulk Extract",
    "SAP Delta Extract",
    "SAP HANA",
    "SAP IDoc",
    "SAP IDoc Extract",
    "SAP IQ",
    "SAP OData",
    "Salesforce API for DataStage",
    "Salesforce.com",
    "Sample",
    "Sequential file",
    "SingleStoreDB",
    "Slowly Changing Dimension",
    "Snowflake",
    "Sort",
    "Split Subrecord",
    "Split Vector",
    "Standardize",
    "Storage volume",
    "Stored Procedure",
    "Surrogate Key Generator",
    "Survive",
    "Switch",
    "Tableau",
    "Tail",
    "Teradata",
    "Teradata database for DataStage",
    "Transformer",
    "Two-source Match",
    "Vertica",
    "Wave Generator",
    "Web Service",
    "Write Range Map",
    "XML Input",
    "XML Output",
    "zOSFile",
]



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import FTP_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthMethod(Enum):
    username_password = "username_password"
    username_password_key = "username_password_key"
    username_key = "username_key"


class ConnectionMode(Enum):
    anonymous = "anonymous"
    basic = "basic"
    mvssftp = "mvssftp"
    sftp = "sftp"
    ftps = "ftps"


class FtpConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "d5dbc62f-7c4c-4d49-8eb2-dab6cef2969c"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    authentication_method: FTP_CONNECTION.AuthMethod | None = Field(None, alias="auth_method")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    connection_mode: FTP_CONNECTION.ConnectionMode = Field(None, alias="connection_mode")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    ftadv_strings: str | None = Field(None, alias="ftadv")
    hostname_or_ip_address: str = Field(None, alias="host")
    key_passphrase: str | None = Field(None, alias="key_passphrase")
    access_mvs_dataset: bool | None = Field(None, alias="mvs_dataset")
    password: str | None = Field(None, alias="password")
    port: int | None = Field(None, alias="port")
    private_key: str | None = Field(None, alias="private_key")
    secure_gateway_id: str | None = Field(None, alias="sg_gateway_id")
    sg_host_original: str | None = Field(None, alias="sg_host_original")
    secure_gateway_as_http_proxy: bool | None = Field(None, alias="sg_http_proxy")
    secure_gateway_security_token: str | None = Field(None, alias="sg_security_token")
    secure_gateway_service_url: str | None = Field(None, alias="sg_service_url")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    sl_host_original: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    port_is_ssl_enabled: bool | None = Field(True, alias="ssl")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    ssl_certificate_hostname: str | None = Field(None, alias="ssl_certificate_host")
    validate_ssl_certificate: bool | None = Field(None, alias="ssl_certificate_validation")
    use_home_as_root: bool | None = Field(True, alias="use_home_as_root")
    username: str = Field("anonymous", alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("ssl_certificate")
            if ((((not self.ssl_certificate_file))) and (self.port_is_ssl_enabled))
            else exclude.add("ssl_certificate")
        )
        (
            include.add("key_passphrase")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                                or (self.connection_mode == "mvssftp")
                            )
                        )
                    )
                    or (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                            or (self.connection_mode == "sftp")
                        )
                    )
                )
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "username_key"
                                )
                                or (self.authentication_method == "username_key")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "username_password_key"
                            )
                            or (self.authentication_method == "username_password_key")
                        )
                    )
                )
            )
            else exclude.add("key_passphrase")
        )
        (
            include.add("satellite_connector_id")
            if (
                (((not self.secure_gateway_id)) and ((not self.satellite_location_id)))
                and (
                    (
                        (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                                or (self.connection_mode == "mvssftp")
                            )
                        )
                    )
                    or (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                            or (self.connection_mode == "sftp")
                        )
                    )
                )
            )
            else exclude.add("satellite_connector_id")
        )
        (
            include.add("ssl_certificate_hostname")
            if (self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_hostname")
        )
        (
            include.add("use_home_as_root")
            if (
                self.connection_mode
                and (
                    (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                    or (self.connection_mode == "sftp")
                )
            )
            else exclude.add("use_home_as_root")
        )
        (
            include.add("private_key")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                                or (self.connection_mode == "mvssftp")
                            )
                        )
                    )
                    or (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                            or (self.connection_mode == "sftp")
                        )
                    )
                )
                and (
                    (
                        (
                            self.authentication_method
                            and (
                                (
                                    hasattr(self.authentication_method, "value")
                                    and self.authentication_method.value == "username_key"
                                )
                                or (self.authentication_method == "username_key")
                            )
                        )
                    )
                    or (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value == "username_password_key"
                            )
                            or (self.authentication_method == "username_password_key")
                        )
                    )
                )
            )
            else exclude.add("private_key")
        )
        (
            include.add("secure_gateway_id")
            if (
                (
                    (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                            or (self.connection_mode == "mvssftp")
                        )
                    )
                )
                or (
                    self.connection_mode
                    and (
                        (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                        or (self.connection_mode == "sftp")
                    )
                )
            )
            else exclude.add("secure_gateway_id")
        )
        (
            include.add("port_is_ssl_enabled")
            if (
                self.connection_mode
                and (
                    (hasattr(self.connection_mode, "value") and self.connection_mode.value == "ftps")
                    or (self.connection_mode == "ftps")
                )
            )
            else exclude.add("port_is_ssl_enabled")
        )
        (
            include.add("access_mvs_dataset")
            if (
                self.connection_mode
                and (
                    (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                    or (self.connection_mode == "mvssftp")
                )
            )
            else exclude.add("access_mvs_dataset")
        )
        (
            include.add("password")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            (
                                (
                                    self.connection_mode
                                    and (
                                        (
                                            hasattr(self.connection_mode, "value")
                                            and self.connection_mode.value == "mvssftp"
                                        )
                                        or (self.connection_mode == "mvssftp")
                                    )
                                )
                            )
                            or (
                                (
                                    self.connection_mode
                                    and (
                                        (
                                            hasattr(self.connection_mode, "value")
                                            and self.connection_mode.value == "sftp"
                                        )
                                        or (self.connection_mode == "sftp")
                                    )
                                )
                            )
                        )
                        and (
                            (
                                (
                                    self.authentication_method
                                    and (
                                        (
                                            hasattr(self.authentication_method, "value")
                                            and self.authentication_method.value == "username_password"
                                        )
                                        or (self.authentication_method == "username_password")
                                    )
                                )
                            )
                            or (
                                (
                                    self.authentication_method
                                    and (
                                        (
                                            hasattr(self.authentication_method, "value")
                                            and self.authentication_method.value == "username_password_key"
                                        )
                                        or (self.authentication_method == "username_password_key")
                                    )
                                )
                            )
                        )
                    )
                    or (
                        (
                            (
                                self.connection_mode
                                and (
                                    (hasattr(self.connection_mode, "value") and self.connection_mode.value == "basic")
                                    or (self.connection_mode == "basic")
                                )
                            )
                        )
                        or (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "ftps")
                                or (self.connection_mode == "ftps")
                            )
                        )
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("authentication_method")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                                or (self.connection_mode == "mvssftp")
                            )
                        )
                    )
                    or (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                            or (self.connection_mode == "sftp")
                        )
                    )
                )
            )
            else exclude.add("authentication_method")
        )
        include.add("ftadv_strings") if (self.access_mvs_dataset) else exclude.add("ftadv_strings")
        (
            include.add("validate_ssl_certificate")
            if (self.port_is_ssl_enabled)
            else exclude.add("validate_ssl_certificate")
        )
        (
            include.add("satellite_location_id")
            if (
                (((not self.secure_gateway_id)) and ((not self.satellite_connector_id)))
                and (
                    (
                        (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "mvssftp")
                                or (self.connection_mode == "mvssftp")
                            )
                        )
                    )
                    or (
                        self.connection_mode
                        and (
                            (hasattr(self.connection_mode, "value") and self.connection_mode.value == "sftp")
                            or (self.connection_mode == "sftp")
                        )
                    )
                )
            )
            else exclude.add("satellite_location_id")
        )
        (
            include.add("username")
            if (
                ((not self.defer_credentials))
                and (
                    (
                        (
                            (
                                (
                                    self.connection_mode
                                    and (
                                        (
                                            hasattr(self.connection_mode, "value")
                                            and self.connection_mode.value == "mvssftp"
                                        )
                                        or (self.connection_mode == "mvssftp")
                                    )
                                )
                            )
                            or (
                                (
                                    self.connection_mode
                                    and (
                                        (
                                            hasattr(self.connection_mode, "value")
                                            and self.connection_mode.value == "sftp"
                                        )
                                        or (self.connection_mode == "sftp")
                                    )
                                )
                            )
                        )
                        and (
                            (
                                (
                                    self.authentication_method
                                    and (
                                        (
                                            hasattr(self.authentication_method, "value")
                                            and self.authentication_method.value == "username_key"
                                        )
                                        or (self.authentication_method == "username_key")
                                    )
                                )
                            )
                            or (
                                (
                                    self.authentication_method
                                    and (
                                        (
                                            hasattr(self.authentication_method, "value")
                                            and self.authentication_method.value == "username_password"
                                        )
                                        or (self.authentication_method == "username_password")
                                    )
                                )
                            )
                            or (
                                (
                                    self.authentication_method
                                    and (
                                        (
                                            hasattr(self.authentication_method, "value")
                                            and self.authentication_method.value == "username_password_key"
                                        )
                                        or (self.authentication_method == "username_password_key")
                                    )
                                )
                            )
                        )
                    )
                    or (
                        (
                            (
                                self.connection_mode
                                and (
                                    (hasattr(self.connection_mode, "value") and self.connection_mode.value == "basic")
                                    or (self.connection_mode == "basic")
                                )
                            )
                        )
                        or (
                            self.connection_mode
                            and (
                                (hasattr(self.connection_mode, "value") and self.connection_mode.value == "ftps")
                                or (self.connection_mode == "ftps")
                            )
                        )
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )

        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")
        (
            include.add("satellite_connector_id")
            if ((not self.secure_gateway_id) and (not self.satellite_location_id))
            and (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            else exclude.add("satellite_connector_id")
        )
        (
            include.add("ftadv_strings")
            if (self.access_mvs_dataset and "true" in str(self.access_mvs_dataset))
            else exclude.add("ftadv_strings")
        )
        (
            include.add("port_is_ssl_enabled")
            if (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "ftps" in str(self.connection_mode.value)
                    )
                    or ("ftps" in str(self.connection_mode))
                )
            )
            else exclude.add("port_is_ssl_enabled")
        )
        (
            include.add("authentication_method")
            if (not self.defer_credentials)
            and (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            else exclude.add("authentication_method")
        )
        (
            include.add("use_home_as_root")
            if (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            else exclude.add("use_home_as_root")
        )
        (
            include.add("key_passphrase")
            if (not self.defer_credentials)
            and (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "username_key" in str(self.authentication_method.value)
                    )
                    or ("username_key" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "username_password_key" in str(self.authentication_method.value)
                    )
                    or ("username_password_key" in str(self.authentication_method))
                )
            )
            else exclude.add("key_passphrase")
        )
        (
            include.add("password")
            if (not self.defer_credentials)
            and (
                (
                    (
                        self.connection_mode
                        and (
                            (
                                hasattr(self.connection_mode, "value")
                                and self.connection_mode.value
                                and "mvssftp" in str(self.connection_mode.value)
                            )
                            or ("mvssftp" in str(self.connection_mode))
                        )
                        and self.connection_mode
                        and (
                            (
                                hasattr(self.connection_mode, "value")
                                and self.connection_mode.value
                                and "sftp" in str(self.connection_mode.value)
                            )
                            or ("sftp" in str(self.connection_mode))
                        )
                    )
                    and (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value
                                and "username_password" in str(self.authentication_method.value)
                            )
                            or ("username_password" in str(self.authentication_method))
                        )
                        and self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value
                                and "username_password_key" in str(self.authentication_method.value)
                            )
                            or ("username_password_key" in str(self.authentication_method))
                        )
                    )
                )
                or (
                    self.connection_mode
                    and (
                        (
                            hasattr(self.connection_mode, "value")
                            and self.connection_mode.value
                            and "basic" in str(self.connection_mode.value)
                        )
                        or ("basic" in str(self.connection_mode))
                    )
                    or self.connection_mode
                    and (
                        (
                            hasattr(self.connection_mode, "value")
                            and self.connection_mode.value
                            and "ftps" in str(self.connection_mode.value)
                        )
                        or ("ftps" in str(self.connection_mode))
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (not self.defer_credentials)
            and (
                (
                    (
                        self.connection_mode
                        and (
                            (
                                hasattr(self.connection_mode, "value")
                                and self.connection_mode.value
                                and "mvssftp" in str(self.connection_mode.value)
                            )
                            or ("mvssftp" in str(self.connection_mode))
                        )
                        and self.connection_mode
                        and (
                            (
                                hasattr(self.connection_mode, "value")
                                and self.connection_mode.value
                                and "sftp" in str(self.connection_mode.value)
                            )
                            or ("sftp" in str(self.connection_mode))
                        )
                    )
                    and (
                        self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value
                                and "username_key" in str(self.authentication_method.value)
                            )
                            or ("username_key" in str(self.authentication_method))
                        )
                        and self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value
                                and "username_password" in str(self.authentication_method.value)
                            )
                            or ("username_password" in str(self.authentication_method))
                        )
                        and self.authentication_method
                        and (
                            (
                                hasattr(self.authentication_method, "value")
                                and self.authentication_method.value
                                and "username_password_key" in str(self.authentication_method.value)
                            )
                            or ("username_password_key" in str(self.authentication_method))
                        )
                    )
                )
                or (
                    self.connection_mode
                    and (
                        (
                            hasattr(self.connection_mode, "value")
                            and self.connection_mode.value
                            and "basic" in str(self.connection_mode.value)
                        )
                        or ("basic" in str(self.connection_mode))
                    )
                    or self.connection_mode
                    and (
                        (
                            hasattr(self.connection_mode, "value")
                            and self.connection_mode.value
                            and "ftps" in str(self.connection_mode.value)
                        )
                        or ("ftps" in str(self.connection_mode))
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("ssl_certificate")
            if (not self.ssl_certificate_file) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate")
        )
        (
            include.add("satellite_location_id")
            if ((not self.secure_gateway_id) and (not self.satellite_connector_id))
            and (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            else exclude.add("satellite_location_id")
        )
        (
            include.add("validate_ssl_certificate")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("validate_ssl_certificate")
        )
        (
            include.add("ssl_certificate_hostname")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_hostname")
        )
        (
            include.add("secure_gateway_id")
            if (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            else exclude.add("secure_gateway_id")
        )
        (
            include.add("ssl_certificate_file")
            if (not self.ssl_certificate) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_file")
        )
        (
            include.add("private_key")
            if (not self.defer_credentials)
            and (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
                and self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "sftp" in str(self.connection_mode.value)
                    )
                    or ("sftp" in str(self.connection_mode))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "username_key" in str(self.authentication_method.value)
                    )
                    or ("username_key" in str(self.authentication_method))
                )
                and self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "username_password_key" in str(self.authentication_method.value)
                    )
                    or ("username_password_key" in str(self.authentication_method))
                )
            )
            else exclude.add("private_key")
        )
        (
            include.add("access_mvs_dataset")
            if (
                self.connection_mode
                and (
                    (
                        hasattr(self.connection_mode, "value")
                        and self.connection_mode.value
                        and "mvssftp" in str(self.connection_mode.value)
                    )
                    or ("mvssftp" in str(self.connection_mode))
                )
            )
            else exclude.add("access_mvs_dataset")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



from .datastage_flow import DataStageFlow, DataStageFlows, DataStageFlowPayloadExtender

# from .datastage_local_subflow import DataStageLocalSubflow
from ibm_watsonx_data_integration.services.datastage.models.stage_names import STAGE_NAMES
from ibm_watsonx_data_integration.services.datastage.models.stage_type_enum import StageTypeEnum
from ibm_watsonx_data_integration.services.datastage.models.stage_type_str import StageTypeStr



from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from pydantic import ConfigDict, Field
from typing import ClassVar


class PostgresqlConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "e1c23729-99d8-4407-b3df-336e33ffdc82"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    discover_data_assets: bool | None = Field(None, alias="auto_discovery")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    database: str = Field(None, alias="database")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    hostname_or_ip_address: str = Field(None, alias="host")
    the_certificate_for_a_key: str | None = Field(None, alias="key_certificate")
    login_timeout: int | None = Field(None, alias="login_timeout")
    password: str = Field(None, alias="password")
    port: int = Field(None, alias="port")
    private_key: str | None = Field(None, alias="private_key")
    proxy: bool | None = Field(False, alias="proxy")
    proxy_host: str = Field(None, alias="proxy_host")
    proxy_password: str | None = Field(None, alias="proxy_password")
    proxy_port: int = Field(None, alias="proxy_port")
    proxy_username: str | None = Field(None, alias="proxy_user")
    query_timeout: int | None = Field(300, alias="query_timeout")
    retry_limit: int | None = Field(2, alias="retry_limit")
    secure_gateway_id: str | None = Field(None, alias="sg_gateway_id")
    sg_host_original: str | None = Field(None, alias="sg_host_original")
    secure_gateway_as_http_proxy: bool | None = Field(None, alias="sg_http_proxy")
    secure_gateway_security_token: str | None = Field(None, alias="sg_security_token")
    secure_gateway_service_url: str | None = Field(None, alias="sg_service_url")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    sl_host_original: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    port_is_ssl_enabled: bool | None = Field(False, alias="ssl")
    ssl_certificate: str | None = Field(None, alias="ssl_certificate")
    ssl_certificate_hostname: str | None = Field(None, alias="ssl_certificate_host")
    validate_ssl_certificate: bool | None = Field(None, alias="ssl_certificate_validation")
    username: str = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    additional_properties: str | None = Field(None, alias="properties")
    ssl_certificate_file: str | None = Field(None, alias="ssl_certificate_file")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("ssl_certificate")
            if ((((not self.ssl_certificate_file))) and (self.port_is_ssl_enabled))
            else exclude.add("ssl_certificate")
        )
        include.add("proxy_port") if (self.proxy) else exclude.add("proxy_port")
        (
            include.add("satellite_connector_id")
            if (((not self.secure_gateway_id)) and (not self.satellite_location_id))
            else exclude.add("satellite_connector_id")
        )
        include.add("proxy_username") if (self.proxy) else exclude.add("proxy_username")
        (
            include.add("ssl_certificate_hostname")
            if (self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_hostname")
        )
        include.add("private_key") if (self.port_is_ssl_enabled) else exclude.add("private_key")
        include.add("proxy_password") if (self.proxy) else exclude.add("proxy_password")
        include.add("password") if (not self.defer_credentials) else exclude.add("password")
        include.add("proxy_host") if (self.proxy) else exclude.add("proxy_host")
        (
            include.add("the_certificate_for_a_key")
            if (((self.port_is_ssl_enabled)) and (self.private_key))
            else exclude.add("the_certificate_for_a_key")
        )
        (
            include.add("validate_ssl_certificate")
            if (self.port_is_ssl_enabled)
            else exclude.add("validate_ssl_certificate")
        )
        (
            include.add("satellite_location_id")
            if (((not self.secure_gateway_id)) and (not self.satellite_connector_id))
            else exclude.add("satellite_location_id")
        )
        include.add("username") if (not self.defer_credentials) else exclude.add("username")
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        include.add("additional_properties") if (self.hidden_dummy_property1) else exclude.add("additional_properties")
        include.add("ssl_certificate_file") if (self.hidden_dummy_property1) else exclude.add("ssl_certificate_file")

        (
            include.add("satellite_connector_id")
            if (not self.secure_gateway_id) and (not self.satellite_location_id)
            else exclude.add("satellite_connector_id")
        )
        include.add("proxy_port") if (self.proxy) else exclude.add("proxy_port")
        include.add("password") if not self.defer_credentials else exclude.add("password")
        include.add("proxy_password") if (self.proxy) else exclude.add("proxy_password")
        include.add("username") if not self.defer_credentials else exclude.add("username")
        (
            include.add("ssl_certificate")
            if (not self.ssl_certificate_file) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate")
        )
        (
            include.add("satellite_location_id")
            if (not self.secure_gateway_id) and (not self.satellite_connector_id)
            else exclude.add("satellite_location_id")
        )
        (
            include.add("the_certificate_for_a_key")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled) and (self.private_key)
            else exclude.add("the_certificate_for_a_key")
        )
        (
            include.add("validate_ssl_certificate")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("validate_ssl_certificate")
        )
        (
            include.add("ssl_certificate_hostname")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_hostname")
        )
        include.add("proxy_username") if (self.proxy) else exclude.add("proxy_username")
        (
            include.add("ssl_certificate_file")
            if (not self.ssl_certificate) and (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("ssl_certificate_file")
        )
        (
            include.add("private_key")
            if (self.port_is_ssl_enabled == "true" or self.port_is_ssl_enabled)
            else exclude.add("private_key")
        )
        include.add("proxy_host") if (self.proxy) else exclude.add("proxy_host")
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module contains models common for all services across CPD."""

from ibm_watsonx_data_integration.cpd_models.access_groups_model import AccessGroup, AccessGroups
from ibm_watsonx_data_integration.cpd_models.account_model import Account, Accounts
from ibm_watsonx_data_integration.cpd_models.connections_model import (
    Connection,
    ConnectionFile,
    ConnectionFiles,
    Connections,
    ConnectionsServiceInfo,
    DatasourceType,
    DatasourceTypeAction,
    DatasourceTypes,
)
from ibm_watsonx_data_integration.cpd_models.job_model import Job, JobRun, JobRuns, JobRunState, Jobs
from ibm_watsonx_data_integration.cpd_models.role_model import Role, Roles, RoleType
from ibm_watsonx_data_integration.cpd_models.service_id_model import ServiceID, ServiceIDs
from ibm_watsonx_data_integration.cpd_models.service_model import Service, Services
from ibm_watsonx_data_integration.cpd_models.trusted_profile_model import TrustedProfile, TrustedProfiles
from ibm_watsonx_data_integration.cpd_models.user_model import UserProfile, UserProfiles, UserSettings
from ibm_watsonx_data_integration.cpd_models.project_model import Project, Projects

__all__ = [
    "Account",
    "Accounts",
    "Project",
    "Projects",
    "Service",
    "Services",
    "UserProfile",
    "UserProfiles",
    "Job",
    "Jobs",
    "UserSettings",
    "JobRun",
    "JobRuns",
    "JobRunState",
    "Role",
    "Roles",
    "RoleType",
    "AccessGroup",
    "AccessGroups",
    "Connection",
    "DatasourceType",
    "DatasourceTypeAction",
    "ConnectionsServiceInfo",
    "ConnectionFile",
    "Connections",
    "DatasourceTypes",
    "ConnectionFiles",
    "ServiceID",
    "ServiceIDs",
    "TrustedProfile",
    "TrustedProfiles",
]



import os
import subprocess
from datetime import datetime
from pathlib import Path


def _format_code(code: str):
    try:
        return subprocess.run(
            [
                "ruff",
                "format",
                "--config",
                Path(__file__).parent.parent / "ruff.toml",
                "-",
            ],
            check=True,
            input=code,
            encoding="utf-8",
            capture_output=True,
        ).stdout
    except subprocess.CalledProcessError as e:
        raise Exception(f"Error while formatting code:\n{code}\n{e.stderr}")


def _generate_flow_name() -> str:
    return f"sdk_generated_flow_{datetime.now().strftime('%Y-%m-%d_%H%M%S')}"


def _generate_test_case_name() -> str:
    return f"sdk_generated_test_case_{datetime.now().strftime('%Y-%m-%d_%H%M%S')}"


def _check_create_dir(dir_path: str):
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)


def _create_init_file(path: str):
    init_file_path = Path(path) / "__init__.py"
    if not init_file_path.exists():
        with open(init_file_path, "w") as f:
            f.write("# This is an empty __init__.py file to make this directory a package.\n")


def _autogenerated_header() -> str:
    """
    Returns a descriptive comment for the top of the autogenerated files
    """
    return "# This code was autogenerated by PythonGenerator\n"
